<?php
/*##################################################
 *
 *   $Id: podcast_cats.class.php 432 2012-03-25 18:53:10Z alain_dev $
 *
 *   begin               	: January 15, 2012
 *   copyright        	    : (C) 2012 Alain GANDON
 *   email               	: alain091@gmail.com
 *
 *   fork from podcast module
 *
###################################################
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
###################################################*/

defined('PHPBOOST') OR exit;

import('content/categories_manager');
require_once PATH_TO_ROOT . '/podcast/podcast_constant.php';

define('PODCAST_DO_NOT_GENERATE_CACHE', false);

class PodcastCats extends CategoriesManager
{
	## Public methods ##

	//Constructor
	function PodcastCats()
	{
		global $Cache, $PODCAST_CATS;
		
		if (!isset($PODCAST_CATS))
			$Cache->load('podcast');
		
		parent::CategoriesManager('podcast_cat', 'podcast', $PODCAST_CATS);
	}

	//Method which removes all subcategories and their content
	function delete_category_recursively($id_cat)
	{
		//We delete the category
		$this->delete_category_with_content($id_cat);
		//Then its content
		foreach ($this->cache_var as $i => $properties)
		{
			if ($i != 0 && $properties['id_parent'] == $i)
			{
				$this->delete_category_recursively($i);
			}
		}

		$this->recount_podcast_per_cat();
	}

	//Method which deletes a category and move its content in another category
	function delete_category_and_move_content($id_category, $new_id_cat_content)
	{
		global $Sql;

		if (!array_key_exists($id_category, $this->cache_var))
		{
			parent::_add_error(NEW_PARENT_CATEGORY_DOES_NOT_EXIST);
			return false;
		}

		parent::delete($id_category);
		foreach ($this->cache_var as $id_cat => $properties)
		{
			if ($id_cat != 0 && $properties['id_parent'] == $id_category)
			{
				parent::move_into_another($id_cat, $new_id_cat_content);
			}
		}

		$Sql->query_inject("UPDATE ".PREFIX."podcast
            SET idcat = " . intval($new_id_cat_content) . "
            WHERE idcat = " . intval($id_category) . "
            LIMIT 1", __LINE__, __FILE__);

		$this->recount_podcast_per_cat();

		return true;
	}
    
    function get_image($mime_type)
    {
        switch ($mime_type)
        {
            case PODCAST_TYPE_MUSIC :
                $image = PATH_TO_ROOT.'/podcast/templates/images/audio.png';
                break;
            case PODCAST_TYPE_VIDEO :
                $image = PATH_TO_ROOT.'/podcast/templates/images/video.png';
                break;
            default:
                $image = PATH_TO_ROOT.'/podcast/podcast.png';
                break;
        }
        return $image;
    }

	//Function which adds a category
	function add($id_parent, $name, $description, $image, $new_auth, $mime_type, $active,
		$itunes_subtitle, $itunes_categories)
	{
		global $Sql,$Cache;

		if (array_key_exists($id_parent, $this->cache_var))
		{
			if (empty($image))
			{
                $image = $this->get_image($mime_type);
			}

			$new_id_cat = parent::add($id_parent, $name);
			$Sql->query_inject("UPDATE ".PREFIX.$this->table."
                SET name = '" . $Sql->escape($name) . "',
                    description = '" . $Sql->escape($description) . "',
                    image = '" . $Sql->escape($image) . "',
                    itunes_subtitle = '" . $Sql->escape($itunes_subtitle) . "',
                    itunes_categories = '" . $Sql->escape($itunes_categories) . "',
                    auth = '" . $Sql->escape($new_auth) . "',
                    mime_type = " . intval($mime_type) . ",
                    active = " . intval($active) . "
                WHERE id = " . intval($new_id_cat) . "
                LIMIT 1", __LINE__, __FILE__);
			//We don't recount the number of questions because this category is empty
			
            $Cache->Generate_module_file('podcast');
			$Cache->load('podcast', true);
			
			return array(true,'e_success');
		}
		else
		{
			return array(false,'e_unexisting_cat');
		}
	}

	//Function which updates a category
	function update($id_cat, $id_parent, $name, $description, $image, $new_auth, $mime_type, $active,
		$itunes_subtitle, $itunes_categories)
	{
		global $Sql, $Cache;

		if (array_key_exists($id_cat, $this->cache_var))
		{
			if ($id_parent != $this->cache_var[$id_cat]['id_parent'])
			{
				if (!parent::move_into_another($id_cat, $id_parent))
				{
					if ($this->check_error(NEW_PARENT_CATEGORY_DOES_NOT_EXIST))
					{
						return array(false,'e_new_cat_does_not_exist');
					}
					if ($this->check_error(NEW_CATEGORY_IS_IN_ITS_CHILDRENS))
					{
						return array(false,'e_infinite_loop');
					}
				}
				else
				{
					$Cache->load('podcast', RELOAD_CACHE);
					$this->recount_podcast_per_cat(PODCAST_DO_NOT_GENERATE_CACHE);
				}
			}
			
			if (empty($image))
			{
                $image = $this->get_image($mime_type);
			}

			$query = "UPDATE ".PREFIX.$this->table."
                SET name = '" . $Sql->escape($name) . "',
                    description = '" . $Sql->escape($description) . "',
                    image = '" . $Sql->escape($image) . "',
                    itunes_subtitle = '" . $Sql->escape($itunes_subtitle) . "',
                    itunes_categories = '" . $Sql->escape($itunes_categories) . "',
                    auth = '" . $Sql->escape($new_auth) . "',
                    mime_type = " . intval($mime_type) . ",
                    active = " . intval($active) . "
                WHERE id = " . intval($id_cat) . "
                LIMIT 1";
			$Sql->query_inject($query, __LINE__, __FILE__);
			
            $Cache->Generate_module_file('podcast');
			$Cache->load('podcast', true);

			return array(true,'e_success');
		}
		else
		{
			return array(false,'e_unexisting_category');
		}
	}

	//Function which moves a category
	function move_into_another($id_cat, $new_id_cat, $position = 0)
	{
		$result = parent::move_into_another($id_cat, $new_id_cat, $position);

		if ($result)
		{
			$this->recount_podcast_per_cat();
		}

		return $result;
	}

	//function which changes the visibility of one category
	function change_visibility($id_cat, $visibility, $generate_cache = LOAD_CACHE)
	{
		$result = parent::change_visibility($id_cat, $visibility, DO_NOT_LOAD_CACHE);

		$this->recount_podcast_per_cat($generate_cache);

		return $result;
	}

	//Recursive function which counts for all category or for one category
	function recount_podcast_per_cat($id_cat = null, $num = null, $generate_cache = true)
	{
		global $Sql, $Cache, $PODCAST_CATS;

		if (!empty($PODCAST_CATS))
		{
			if (!function_exists('array_fill_keys'))
			{
				$num_podcast = array();
   				$array_keys = array_keys($PODCAST_CATS);
   				
   				foreach ($array_keys as $idkey)
   				{
   					$num_podcast[$idkey] = 0;
   				}
			}
			else
			{
				$num_podcast = array_fill_keys(array_keys($PODCAST_CATS), 0);
			}
		}

		if (empty($id_cat))
		{
			$result = $Sql->query_while("SELECT id, idcat
                FROM ".PREFIX."podcast
                WHERE infos = " . intval(PODCAST_STATUS_APROBED) . "
                ORDER BY idcat", __LINE__, __FILE__);

			while ($row = $Sql->fetch_assoc($result))
			{
				if (!empty($PODCAST_CATS[$row['idcat']]))
				{
					$num_podcast[$row['idcat']]++;
				}
			}

			$Sql->query_close($result);

			if (!empty($PODCAST_CATS[0]) AND $PODCAST_CATS[0]['num_podcast'] != $num_podcast[0])
			{
                $this->save_config_num_podcast($num_podcast[0]);
			}

			if (!empty($num_podcast))
			{
				foreach ($num_podcast as $key => $value)
				{
					if ($key != 0 AND $PODCAST_CATS[$key] != $value)
					{
                        $this->save_num_podcast($key,$value);
					}
				}
			}
		}
		else
		{
			if (is_null($num))
			{
				$num = (int) $Sql->query("SELECT COUNT(1)
                    FROM ".PREFIX."podcast
                    WHERE idcat = " . intval($id_cat) . "
                    AND infos = " . intval(PODCAST_STATUS_APROBED), __LINE__, __FILE__);
			}

			if ($id_cat > 0)
			{
                $this->save_num_podcast($id_cat,$num);
			}
			else
			{
                $this->save_config_num_podcast($num);
			}
		}

		if ($generate_cache)
		{
			$Cache->generate_module_file('podcast');
			$Cache->load('podcast',true); // force refreshing the cache global variables
		}

		return true;
	}
    
	/**
	 * return formated url for category
	 * @param int id of category
	 * @param string name of category
	 * @param string query string to append if any
	 * @return path part of url
	 */
    function get_cat_link($id_cat,$name,$qs='')
    {
		global $podcast_httprequest_cat_config;
		
		$url = url(
			sprintf($podcast_httprequest_cat_config['url']['unrewrited'],intval($id_cat)),
			sprintf($podcast_httprequest_cat_config['url']['rewrited'],intval($id_cat),url_encode_rewrite($name))
			);
		if (!empty($qs))
		{
			$url .= (strpos($url,'?')!==false) ? '&amp;' : '?';
			$url .= $qs;
		}
		return $url;
    }
	/**
	 * return short formated url for category
	 * @param int id of category
	 * @param string query string to append if any
	 * @return path part of url
	 */
    function get_cat_short_link($id_cat,$qs='')
    {
		$url = url(
			'podcast.php?cat='.intval($id_cat),
			'podcast-0-'.intval($id_cat).'.php'
			);
		if (!empty($qs))
		{
			$url .= (strpos($url,'?')!==false) ? '&amp;' : '?';
			$url .= $qs;
		}
		return $url;
	}
	/**
	 * return formated url for podcast item
	 * @param int id of item
	 * @param int idcat id of category
	 * @param string qs query string to append
	 * @return path part of url
	 */	
    function get_item_link($id,$id_cat,$name,$qs='')
    {
		global $podcast_httprequest_config, $podcast_httprequest_2_config;
		
		if (!empty($id) AND !empty($id_cat))
		{
			$url = url(
				sprintf($podcast_httprequest_2_config['url']['unrewrited'],intval($id),intval($id_cat)),
				sprintf($podcast_httprequest_2_config['url']['rewrited'],intval($id),intval($id_cat),url_encode_rewrite($name))
				);
		}
		else
		{
			$url = url(
				sprintf($podcast_httprequest_config['url']['unrewrited'],intval($id)),
				sprintf($podcast_httprequest_config['url']['rewrited'],intval($id),url_encode_rewrite($name))
				);
		}
		if (!empty($qs))
		{
			$url .= (strpos($url,'?')!==false) ? '&amp;' : '?';
			$url .= $qs;
		}
		return $url;
    }
	/**
	 * return formated url for an action
	 * @param array $actions array of key => value
	 * @param bool $token append a token key or not
	 * @param string $base basename of php script
	 * @return path of url
	 */		
    function get_action_link($actions, $token=true, $base='podcast_action')
    {
		global $Session;
		
		if (!empty($actions))
		{
			$params = array();
			foreach ($actions as $key => $value)
				$params[] = $key.'='.$value;
			$str = $base.'.php?'.implode('&',$params);
		}
		else
			$str = $base.'.php';
			
		if (!empty($token))
		{
			$str .= (strpos($str,'?') !== false) ? '&' : '?';
			$str .= 'token='.$Session->get_token();
		}
		return url($str);
    }
	/**
	 * build breadcrumb
	 * @param int $id_cat category id
	 * @return breadcrumb object
	 */	
	function bread_crumb($id_cat)
	{
		global $Bread_crumb, $PODCAST_CATS;
		
		if (empty($PODCAST_CATS[$id_cat])) return;

		$Bread_crumb->add($PODCAST_CATS[$id_cat]['name'], $this->get_cat_link($id_cat,$PODCAST_CATS[$id_cat]['name']));

		$id_parent = $PODCAST_CATS[$id_cat]['id_parent'];		
		while ($id_parent > 0)
		{
			$Bread_crumb->add($PODCAST_CATS[$id_parent]['name'], $this->get_cat_link($id_parent,$PODCAST_CATS[$id_parent]['name']));
			$id_parent = $PODCAST_CATS[$id_parent]['id_parent'];
		}
		$Bread_crumb->reverse();
	}
   	/**
	 * get a podcast record
	 * @param int $id podcast id
	 * @return array of $fields => $value
	 */	
	function get($id)
	{
		global $Sql;
		
		$query = "SELECT p.*, mb.login, mb.level
			FROM " . PREFIX . "podcast AS p
			LEFT JOIN " . DB_TABLE_MEMBER . " AS mb ON (p.iduser = mb.user_id)
			WHERE id = " . intval($id);
			
		$result = $Sql->query_while($query, __LINE__, __FILE__);
		$podcast = $Sql->fetch_assoc($result);
		$Sql->query_close($result);
		
		return $podcast;
	}
	
    function get_all_cats($id_cat, $sort='', $mode='', &$pagination=false)
    {
        global $Sql, $PODCAST_CONFIG;
        
		$query = "SELECT p.*, mb.login, mb.level
			FROM " . PREFIX . "podcast AS p
			LEFT JOIN " . DB_TABLE_MEMBER . " AS mb ON p.iduser = mb.user_id
			WHERE idcat = " . intval($id_cat) ."
			AND infos = ".intval(PODCAST_STATUS_APROBED);
            
        if (!empty($sort) AND !empty($mode))
            $query .= " ORDER BY " . $sort . " " . $mode ;
            
        if (!empty($pagination))
        {
            $query .= ' '.$Sql->limit($pagination->get_first_msg($PODCAST_CONFIG['pagin'], 'p'), $PODCAST_CONFIG['pagin']);
        }
        
        $result = $Sql->query_while($query, __FILE__, __LINE__);
        return $result;
    }
    
    function check_value($field, $value)
    {
		if ($field < 0) return '';
        return ($field == $value) ? ' checked="checked"' : '';
    }
    
    function check_bit($field, $bit)
    {
		if ($field < 0) return '';
        return (($field & $bit) != 0) ? ' checked="checked"' : '';
    }
    
    function insert_id()
    {
        global $Sql;
        return $Sql->insert_id('');
    }

	//method which deletes a category and its content (not recursive)
	protected function delete_category_with_content($id)
	{
		global $Sql;

		//If the category is successfully deleted
		if (parent::delete($id))
		{
			//We remove its whole content
			$Sql->query_inject("DELETE
                FROM ".PREFIX."podcast
                WHERE idcat = " . intval($id), __LINE__, __FILE__);
			return true;
		}
		else
		{
			return false;
		}
	}
    
    protected function save_config_num_podcast($num)
    {
        global $Sql;
        
        $query = "SELECT value
            FROM ".PREFIX."configs
            WHERE name='podcast'
            LIMIT 1";

        $result = $Sql->query_while($query,__LINE__, __FILE__);
        $config = $Sql->fetch_assoc($result);
        $config = unserialize($config['value']);
        $config['root']['num_podcast'] = intval($num);

        $Sql->query_inject("UPDATE ".PREFIX."configs
            SET value = '" . $Sql->escape(serialize($config)) . "'
            WHERE name = 'podcast'
            LIMIT 1", __LINE__, __FILE__);
    }
    
    protected function save_num_podcast($id_cat,$num)
    {
        global $Sql;
        
        $Sql->query_inject("UPDATE ".PREFIX.$this->table."
            SET num_podcast = " . intval($num) . "
            WHERE id = " . intval($id_cat) ."
            LIMIT 1", __LINE__, __FILE__);
    }
}

?>